@@ -192,6 +192,7 @@ stanmarg_data <- function(YX = NULL, S = NULL, N, Ng, grpnum, # data
                           gam_sign, b_sign, psi_r_sign, phi_r_sign,
                           lavpartable = NULL, # for priors
                           dumlv = NULL, # for sampling lvs
+                          wigind = NULL, # wiggle indicator
                           ...) {
   
   dat <- list()
@@ -233,7 +234,6 @@ stanmarg_data <- function(YX = NULL, S = NULL, N, Ng, grpnum, # data
     }
   }
 
-  if(dat$missing & save_lvs) stop("blavaan ERROR: lvs cannot currently be saved when data are missing.")
   dat$save_lvs <- save_lvs
 
   dat$p <- dim(Lambda_y_skeleton)[2]
@@ -250,13 +250,13 @@ stanmarg_data <- function(YX = NULL, S = NULL, N, Ng, grpnum, # data
   dat$q <- dim(Lambda_x_skeleton)[2]
   dat$n <- dim(Lambda_x_skeleton)[3]
   tmpres <- group_sparse_skeleton(Lambda_x_skeleton)
-  dat$len_w2 <- max(tmpres$g_len)
-  dat$w2 <- tmpres$w
-  dat$v2 <- tmpres$v
-  dat$u2 <- tmpres$u
-  dat$wg2 <- array(tmpres$g_len, length(tmpres$g_len))
-  dat$w2skel <- w2skel
-  dat$lam_x_sign <- lam_x_sign
+  #dat$len_w2 <- max(tmpres$g_len)
+  #dat$w2 <- tmpres$w
+  #dat$v2 <- tmpres$v
+  #dat$u2 <- tmpres$u
+  #dat$wg2 <- array(tmpres$g_len, length(tmpres$g_len))
+  #dat$w2skel <- w2skel
+  #dat$lam_x_sign <- lam_x_sign
 
   tmpres <- group_sparse_skeleton(Gamma_skeleton)
   dat$len_w3 <- max(tmpres$g_len)
@@ -364,21 +364,21 @@ stanmarg_data <- function(YX = NULL, S = NULL, N, Ng, grpnum, # data
     dPhi[g,,] <- tmpmat
   }
   tmpres <- group_sparse_skeleton(dPhi)
-  dat$len_w11 <- max(tmpres$g_len)
-  dat$w11 <- tmpres$w
-  dat$v11 <- tmpres$v
-  dat$u11 <- tmpres$u
-  dat$wg11 <- array(tmpres$g_len, length(tmpres$g_len))
-  dat$w11skel <- w11skel
+  #dat$len_w11 <- max(tmpres$g_len)
+  #dat$w11 <- tmpres$w
+  #dat$v11 <- tmpres$v
+  #dat$u11 <- tmpres$u
+  #dat$wg11 <- array(tmpres$g_len, length(tmpres$g_len))
+  #dat$w11skel <- w11skel
 
   tmpres <- group_sparse_skeleton(Phi_r_skeleton)
-  dat$len_w12 <- max(tmpres$g_len)
-  dat$w12 <- tmpres$w
-  dat$v12 <- tmpres$v
-  dat$u12 <- tmpres$u
-  dat$wg12 <- array(tmpres$g_len, length(tmpres$g_len))
-  dat$w12skel <- w12skel
-  dat$phi_r_sign <- phi_r_sign
+  #dat$len_w12 <- max(tmpres$g_len)
+  #dat$w12 <- tmpres$w
+  #dat$v12 <- tmpres$v
+  #dat$u12 <- tmpres$u
+  #dat$wg12 <- array(tmpres$g_len, length(tmpres$g_len))
+  #dat$w12skel <- w12skel
+  #dat$phi_r_sign <- phi_r_sign
 
   if(dat$has_data & is.null(Nu_skeleton)) stop("blavaan ERROR: Nu_skeleton not provided")
   tmpres <- group_sparse_skeleton(Nu_skeleton)
@@ -399,7 +399,8 @@ stanmarg_data <- function(YX = NULL, S = NULL, N, Ng, grpnum, # data
 
   ## priors; first make sure they match what is in the stan file
   check_priors(lavpartable)
-
+  dat$wigind <- wigind
+  
   pris <- format_priors(lavpartable, "lambda")
   dat$lambda_y_mn <- pris[['p1']]; dat$lambda_y_sd <- pris[['p2']]
   dat$len_lam_y <- length(dat$lambda_y_mn)
