@@ -59,5 +59,15 @@ test_that("blavaan arguments", {
   
   ## unknown prior
   expect_error(bsem(model, data=Data, dp=dpriors(psi="mydist(1,.5)")))
-  
+
+  ## wiggle argument
+  expect_error(bsem(model, data=Data, wiggle='a', wiggle.sd=0))
+  expect_error(bsem(model, data=Data, wiggle='sponge'))
+
+  HS.model <- ' visual  =~ x1 + x2 + x3 '
+
+  expect_s4_class(bcfa(HS.model, data=HolzingerSwineford1939, target="stan", do.fit=FALSE, group="school", group.equal=c("intercepts","loadings"), wiggle=c("intercepts"), wiggle.sd=.1), "blavaan")
+  expect_s4_class(bcfa(HS.model, data=HolzingerSwineford1939, target="stanclassic", do.fit=FALSE, group="school", group.equal=c("intercepts","loadings"), wiggle=c("intercepts"), wiggle.sd=.1), "blavaan")
+  expect_s4_class(bcfa(HS.model, data=HolzingerSwineford1939, target="jags", do.fit=FALSE, group="school", group.equal=c("intercepts","loadings"), wiggle=c("intercepts"), wiggle.sd=.1), "blavaan")
+                  
 })
